/*
 * model.h (version of the bounded rational CSR model)
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define Random	100	// No. of random seeds
#define NFirm	100	// No. of firms
#define NWorker	10000	// No. of workers
#define NConsumer 10000	// No. of consumers
#define NHolder	10000	// No. of shareholders
#define Memory	10	// Capacity of Memory List
#define HalfMemory 5	// No. of replacable quantity of memory list (= Memory / 2)
#define Time	100	// Simulation periods
#define Noise	0.1	// Cognitive probability

typedef struct _list{	// Memory list
	int firm[Memory];	// Firm number
} List;

typedef struct _env{
	double upcsr;	// s1, Upper limit of CSR activity in firms performance
	double depr;	// s3, Depreciation Ratio = 0.1
	double beta;	// s2, Ratio of labor force to stock on production function
	double iner;	// s5, Inertia quantity for value functions for firm
	double eff;	// s4, Standard level of effort
	double scsr;	// s6, Upper limit of shakeholders CSR awareness
	double abl;	// Standard level of ability
	double ablr;	// Range of ability
	double iunit;	// Investment unit
} Env;

typedef struct _firm{
	double csr;	// Degree of CSR [0,1)
	double quantity;	// Product Quantity
	double wage;	// Wage
	double demand;	// Demand of Goods
	double invest;	// Investment
	double stock;	// Capital Stock
} Firm;

typedef struct _worker{
	double abl;	// Ability
	double csr;	// Awareness for CSR [0,1)
	double effort;	// Effort Level
	double wage;	// Wage
	int choice;	// Firm No to be hired
	List list;	// Candidate List
} Worker;

typedef struct _consumer{
	double csr;	// Awareness for CSR [0,1) 
	int choice;	// Firm No to be hired
	List list;	// Candidate List
} Consumer;

typedef struct _holder{
	double csr;	// Awareness for CSR [0,1)
	int choice;	// Firm No. to invest
	List list;	// Candidate List
} Holder;

/* init.c */
void ini_agent(void);	// Initialising Agents
void initialize(int);	// All initialisings
void ini_ran(void);	// Initialising observation procedures
void obsF(int);		// Calculations for observation
void printResF(void);	// Output

/* phase.c */
void preF(void);		// Preprocess
void investF(void);	// Investment of Shareholders
void jobF(void);		// Job Marketing
void productF(void);	// Product by firms
void consumpF(void);	// Consumption
void updateList(void);	// Update List of Memory
void cutCsrF(void);	// Cutting CSR awareness

/* funct.c */
double maxF(double,double);
void orderF(int,double [],int []);
double noiseF(double,double,double,double);

/* ran1.c */
double ran1(long *);	// Random generator [0,1)

